@@ -1,5 +1,5 @@
 # Licensed under the Apache License: http://www.apache.org/licenses/LICENSE-2.0
-# For details: https://github.com/nedbat/coveragepy/blob/master/NOTICE.txt
+# For details: https://github.com/coveragepy/coveragepy/blob/main/NOTICE.txt
 
 """A simple Python template renderer, for a nano-subset of Django syntax.
 
@@ -13,19 +13,19 @@ http://aosabook.org/en/500L/a-template-engine.html
 from __future__ import annotations
 
 import re
-
-from typing import (
-    Any, Callable, Dict, List, NoReturn, Optional, Set, Union, cast,
-)
+from collections.abc import Callable
+from typing import Any, NoReturn, cast
 
 
 class TempliteSyntaxError(ValueError):
     """Raised when a template has a syntax error."""
+
     pass
 
 
 class TempliteValueError(ValueError):
     """Raised when an expression won't evaluate in a template."""
+
     pass
 
 
@@ -33,7 +33,7 @@ class CodeBuilder:
     """Build source code conveniently."""
 
     def __init__(self, indent: int = 0) -> None:
-        self.code: List[Union[str, CodeBuilder]] = []
+        self.code: list[str | CodeBuilder] = []
         self.indent_level = indent
 
     def __str__(self) -> str:
@@ -53,7 +53,7 @@ class CodeBuilder:
         self.code.append(section)
         return section
 
-    INDENT_STEP = 4      # PEP8 says so!
+    INDENT_STEP = 4  # PEP8 says so!
 
     def indent(self) -> None:
         """Increase the current indent for following lines."""
@@ -63,14 +63,14 @@ class CodeBuilder:
         """Decrease the current indent for following lines."""
         self.indent_level -= self.INDENT_STEP
 
-    def get_globals(self) -> Dict[str, Any]:
+    def get_globals(self) -> dict[str, Any]:
         """Execute the code, and return a dict of globals it defines."""
         # A check that the caller really finished all the blocks they started.
         assert self.indent_level == 0
         # Get the Python source as a single string.
         python_source = str(self)
         # Execute the source, defining globals, and return them.
-        global_namespace: Dict[str, Any] = {}
+        global_namespace: dict[str, Any] = {}
         exec(python_source, global_namespace)
         return global_namespace
 
@@ -90,6 +90,10 @@ class Templite:
 
         {% if var %}...{% endif %}
 
+    if-else::
+
+        {% if var %}...{% else %}...{% endif %}
+
     Comments are within curly-hash markers::
 
         {# This will be ignored #}
@@ -109,15 +113,16 @@ class Templite:
                 <p>You are interested in {{topic}}.</p>
             {% endif %}
             ''',
-            {'upper': str.upper},
+            {"upper": str.upper},
         )
         text = templite.render({
-            'name': "Ned",
-            'topics': ['Python', 'Geometry', 'Juggling'],
+            "name": "Ned",
+            "topics": ["Python", "Geometry", "Juggling"],
         })
 
     """
-    def __init__(self, text: str, *contexts: Dict[str, Any]) -> None:
+
+    def __init__(self, text: str, *contexts: dict[str, Any]) -> None:
         """Construct a Templite with the given `text`.
 
         `contexts` are dictionaries of values to use for future renderings.
@@ -128,8 +133,8 @@ class Templite:
         for context in contexts:
             self.context.update(context)
 
-        self.all_vars: Set[str] = set()
-        self.loop_vars: Set[str] = set()
+        self.all_vars: set[str] = set()
+        self.loop_vars: set[str] = set()
 
         # We construct a function in source form, then compile it and hold onto
         # it, and execute it to render the template.
@@ -143,7 +148,7 @@ class Templite:
         code.add_line("extend_result = result.extend")
         code.add_line("to_str = str")
 
-        buffered: List[str] = []
+        buffered: list[str] = []
 
         def flush_output() -> None:
             """Force `buffered` to the code builder."""
@@ -161,49 +166,54 @@ class Templite:
         squash = in_joined = False
 
         for token in tokens:
-            if token.startswith('{'):
+            if token.startswith("{"):
                 start, end = 2, -2
-                squash = (token[-3] == '-')
+                squash = (token[-3] == "-")  # fmt: skip
                 if squash:
                     end = -3
 
-                if token.startswith('{#'):
+                if token.startswith("{#"):
                     # Comment: ignore it and move on.
                     continue
-                elif token.startswith('{{'):
+                elif token.startswith("{{"):
                     # An expression to evaluate.
                     expr = self._expr_code(token[start:end].strip())
                     buffered.append("to_str(%s)" % expr)
                 else:
-                    # token.startswith('{%')
+                    # token.startswith("{%")
                     # Action tag: split into words and parse further.
                     flush_output()
 
                     words = token[start:end].strip().split()
-                    if words[0] == 'if':
+                    if words[0] == "if":
                         # An if statement: evaluate the expression to determine if.
                         if len(words) != 2:
                             self._syntax_error("Don't understand if", token)
-                        ops_stack.append('if')
+                        ops_stack.append("if")
                         code.add_line("if %s:" % self._expr_code(words[1]))
                         code.indent()
-                    elif words[0] == 'for':
+                    elif words[0] == "else":
+                        if len(words) != 1:
+                            self._syntax_error("Don't understand else", token)
+                        if not ops_stack or ops_stack[-1] != "if":
+                            self._syntax_error("Mismatched else", token)
+                        code.dedent()
+                        code.add_line("else:")
+                        code.indent()
+                    elif words[0] == "for":
                         # A loop: iterate over expression result.
-                        if len(words) != 4 or words[2] != 'in':
+                        if len(words) != 4 or words[2] != "in":
                             self._syntax_error("Don't understand for", token)
-                        ops_stack.append('for')
+                        ops_stack.append("for")
                         self._variable(words[1], self.loop_vars)
                         code.add_line(
-                            "for c_{} in {}:".format(
-                                words[1],
-                                self._expr_code(words[3])
-                            )
+                            f"for c_{words[1]} in {self._expr_code(words[3])}:",
                         )
                         code.indent()
-                    elif words[0] == 'joined':
-                        ops_stack.append('joined')
+                    elif words[0] == "joined":
+                        ops_stack.append("joined")
                         in_joined = True
-                    elif words[0].startswith('end'):
+                    elif words[0].startswith("end"):
                         # Endsomething.  Pop the ops stack.
                         if len(words) != 1:
                             self._syntax_error("Don't understand end", token)
@@ -213,7 +223,7 @@ class Templite:
                         start_what = ops_stack.pop()
                         if start_what != end_what:
                             self._syntax_error("Mismatched end tag", end_what)
-                        if end_what == 'joined':
+                        if end_what == "joined":
                             in_joined = False
                         else:
                             code.dedent()
@@ -236,14 +246,14 @@ class Templite:
         for var_name in self.all_vars - self.loop_vars:
             vars_code.add_line(f"c_{var_name} = context[{var_name!r}]")
 
-        code.add_line('return "".join(result)')
+        code.add_line("return ''.join(result)")
         code.dedent()
         self._render_function = cast(
             Callable[
-                [Dict[str, Any], Callable[..., Any]],
-                str
+                [dict[str, Any], Callable[..., Any]],
+                str,
             ],
-            code.get_globals()['render_function'],
+            code.get_globals()["render_function"],
         )
 
     def _expr_code(self, expr: str) -> str:
@@ -268,7 +278,7 @@ class Templite:
         """Raise a syntax error using `msg`, and showing `thing`."""
         raise TempliteSyntaxError(f"{msg}: {thing!r}")
 
-    def _variable(self, name: str, vars_set: Set[str]) -> None:
+    def _variable(self, name: str, vars_set: set[str]) -> None:
         """Track that `name` is used as a variable.
 
         Adds the name to `vars_set`, a set of variable names.
@@ -280,7 +290,7 @@ class Templite:
             self._syntax_error("Not a valid name", name)
         vars_set.add(name)
 
-    def render(self, context: Optional[Dict[str, Any]] = None) -> str:
+    def render(self, context: dict[str, Any] | None = None) -> str:
         """Render this template by applying it to `context`.
 
         `context` is a dictionary of values to use in this rendering.
@@ -302,7 +312,7 @@ class Templite:
                     value = value[dot]
                 except (TypeError, KeyError) as exc:
                     raise TempliteValueError(
-                        f"Couldn't evaluate {value!r}.{dot}"
+                        f"Couldn't evaluate {value!r}.{dot}",
                     ) from exc
             if callable(value):
                 value = value()
